# -*- coding:utf-8 -*-
# @Time: 2020/11/10 13:30
# @Author: caiyucheng
# @File: cross_n_min_amount.py

import pandas as pd
import numpy as np


def cal(high, low, n=20):
    """
    二十日最高价 / 二十日最低价 取对数
    Parameters
    ----------
    high: pd.Series
          股票最高价
    low: pd.Series
         股票最低价
    n: int
       计算区间

    Returns
    -------
    pd.Series
        Examples
        --------
        0        NaN
        1        NaN
        2        NaN
        3        NaN
        4        NaN
        5        NaN
        6        NaN
        7        NaN
        8        NaN
        9        NaN
        10       NaN
        11       NaN
        12       NaN
        13       NaN
        14       NaN
        15       NaN
        16       NaN
        17       NaN
        18       NaN
        19    0.0227
        20    0.0227
        21    0.0227
        22    0.0193
        23    0.0300
        24    0.0300
        25    0.0300
        26    0.0300
        27    0.0300
        28    0.0246
        29    0.0158
        Name: x, dtype: float64
        -----------------------
    """
    if isinstance(high, pd.Series) and isinstance(low, pd.Series) and isinstance(n, int):
        high = high.rolling(window=n).max()
        low = low.rolling(window=n).max()
        ln_lh = (high / low).apply(np.log)
        ln_lh[np.isinf(ln_lh)] = np.nan
        return ln_lh
    else:
        raise ValueError('请输入格式正确的参数, high, low为Series, n为int')


# -*- coding:utf-8 -*-
# @Time: 2020/11/9 08:56
# @Author: jiangwenhao
# @File: firstcrossttime.py

import pandas as pd
import numpy as np


def cal(close, up_limit, n=1) -> pd.Series:
	"""
	计算个股突破涨停价格的分钟数

	Parameters
	----------
	close : Series
		收盘价
	up_limit : Series
		涨停价（index应与close对应）
	n : int
		第几次突破
	Returns
	-------
	pd.Series or np.array
		个股突破涨停价格的分钟数
		Examples:
			----------------------------------
			120
			----------------------------------
	"""
	if isinstance(close, pd.Series) and isinstance(up_limit, pd.Series):
		df_compare = pd.DataFrame({'close': close, 'up_limit': up_limit}).reset_index(drop=True)
		df_compare['last_close'] = df_compare['close'].shift(1)
		df_compare['is_cross'] = df_compare.apply(
			lambda x: 1 if
			(round(x['close'], 2) >= round(x['up_limit'], 2)) & (round(x['last_close'], 2) < round(x['up_limit'], 2))
			else 0, axis=1
		)
		cross_time = df_compare[df_compare['is_cross'] == 1].index.to_list()
		if len(cross_time) >= n:
			n_cross_time = cross_time[n-1]
		else:
			n_cross_time = 2
	else:
		raise ValueError('传入的close和up_limit需为Series格式')
	return n_cross_time



# -*- coding:utf-8 -*-
# @Time: 2020/11/10 13:30
# @Author: caiyucheng
# @File: cross_n_min_amount.py

import pandas as pd
import numpy as np


def cal(close, n, m):
    """
    计算股票n期累积收益率与m期累积收益率的差(价格动量差), 一般来说用 短期收益率-长期收益率 所以最好 n > m
    Parameters
    ----------
    close: Series
           股票收盘价
    n: int
       n期累计收益率
    m: int
       m期累计收益率
    Returns
    -------
    pd.Series
    股票n期累积收益率与m期累积收益率的差
    Examples
    -----------
        0       NaN
        1       NaN
        2       NaN
        3       NaN
        4       NaN
        5   -0.0162
        6    0.0303
        7    0.0166
        8    0.0269
        9   -0.0193
        Name: x, dtype: float64
        -----------------------
    """
    if isinstance(close, pd.Series) and isinstance(n, int) and isinstance(m, int):
        close_n = close / close.shift(n) - 1
        close_m = close / close.shift(m) - 1
        return close_n - close_m
    else:
        raise ValueError('请输入参数的正确格式, close为Series, n, m为int')


# -*- coding:utf-8 -*-
# @Time: 2020/11/10 13:30
# @Author: caiyucheng
# @File: cross_n_min_amount.py

import pandas as pd
import numpy as np


def cal(stock_close, index_close, n, m=1):
    """
    计算n期内股票与指数m期收益率皮尔逊相关系数, m默认为1
    Parameters
    ----------
    stock_close：Series
                 股票收盘价
    index_close: Series
                 指数收盘价
    n: int
       计算皮尔逊系数区间
    m: 计算收益率区间 默认为1
    Returns
    -------
    pd.Series
        n周期内股票与指数的m日收益率的皮尔逊相关系数
        Examples
        ----------
            0       NaN
            1       NaN
            2       NaN
            3       NaN
            4       NaN
            5       NaN
            6    0.0467
            7    0.4038
            8    0.4725
            9    0.4891
            Name: x, dtype: float64
            -----------------------
    """
    if isinstance(stock_close, pd.Series) and isinstance(index_close, pd.Series) and isinstance(m, int) & isinstance(n, int):
        if len(stock_close) != len(index_close):
            raise Exception('请对齐股票与指数日期, 最好检查股票期间是否有停牌日, 如果有停牌最好按指数日期左对齐merge之后再进该函数')
        list_test = list(range(len(stock_close)))
        stock_close = stock_close / stock_close.shift(m) - 1
        index_close = index_close / index_close.shift(m) - 1
        pearson_cor = [__correlation(index_ret=index_close, stock_ret=stock_close, index_=index, period=n) for index in list_test]
        pearson_cor = pd.Series(pearson_cor)
    else:
        raise ValueError('传入的stock_close, index_close需为Series格式,n, m需为int格式')
    return pearson_cor


def __correlation(index_ret,  stock_ret, index_, period):
    if index_ - period <= 0:
        return np.nan
    a = index_
    b = index_ - period
    x = stock_ret[b: a]
    y = index_ret[b: a]
    meanX = x.mean()
    deviationX = x.std(ddof=0)
    stardardizedX = (x - meanX) / deviationX
    meanY = y.mean()
    deviationY = y.std(ddof=0)
    stardardizedY = (y - meanY) / deviationY
    return (stardardizedX * stardardizedY).mean()


# -*- coding:utf-8 -*-
# @Time: 2020/11/8 16:44
# @Author: jiangwenhao
# @File: riseovernum_nday.py

import pandas as pd
import numpy as np


def cal(close, up_limit, ma=5):
	"""
	计算个股ma日内涨停数

	Parameters
	----------
	close : Series
		收盘价
	up_limit : Series
		涨停价（index应与close对应）
	ma: int
		日期区间(ma日）
	Returns
	-------
	pd.Series or np.array
		个股ma日内涨停数
		Examples:
			----------------------------------
			0      NaN
			1      NaN
			2      NaN
			3      NaN
			4      0.0
			...
			120    1.0
			121    1.0
			122    1.0
			123    1.0
			124    0.0
			Name: riseovernum_nday, Length: 125, dtype: float64
			----------------------------------
	"""
	if isinstance(close, pd.Series) and isinstance(up_limit, pd.Series) and isinstance(ma, int):
		df_compare = pd.DataFrame({'close': close, 'up_limit': up_limit}).reset_index(drop=True)
		df_compare['is_uplimit'] = 0
		df_compare.loc[round(df_compare['close'], 2) == round(df_compare['up_limit'], 2), 'is_uplimit'] = 1
		riseovernum_nday = df_compare['is_uplimit'].rolling(window=ma).sum()
	else:
		raise ValueError('传入的close和up_limit需为Series格式,ma需要为int格式')
	return riseovernum_nday



# -*- coding:utf-8 -*-
# @Time: 2020/11/10 13:30
# @Author: caiyucheng
# @File: cross_n_min_amount.py

import pandas as pd
import numpy as np
import talib


def cal(target, fastperiod_num=12, slowperiod_num=26, signalperiod_num=9):
    """
    计算目标值的macd 周期为默认参数(通常软件也是这么计算macd)
    Parameters
    ----------
    target: pd.Series
            目标值随意, 一般为close 或者 vol
    fastperiod_num: int
    slowperiod_num: int
    signalperiod_num: int
    Returns
    -------
    pd.Series
        Examples
        --------
        0        NaN
        1        NaN
        2        NaN
        3        NaN
        4        NaN
        5        NaN
        6        NaN
        7        NaN
        8        NaN
        9        NaN
        10       NaN
        11       NaN
        12       NaN
        13       NaN
        14       NaN
        15       NaN
        16       NaN
        17       NaN
        18       NaN
        19       NaN
        20       NaN
        21       NaN
        22       NaN
        23       NaN
        24       NaN
        25       NaN
        26       NaN
        27       NaN
        28       NaN
        29       NaN
        30       NaN
        31       NaN
        32       NaN
        33    0.4401
        34    0.4021
        35    0.2905
        36    0.2528
        37    0.2170
        38    0.1888
        39    0.2596
        40    0.2528
        41    0.1691
        42    0.1549
        43    0.1360
        44    0.1225
        45    0.0938
        46   -0.0211
        47   -0.1290
        48   -0.2726
        49   -0.4023
        Name: x, dtype: float64
        -----------------------
    """
    if isinstance(target, pd.Series) and isinstance(fastperiod_num, int) and isinstance(slowperiod_num, int) and isinstance(signalperiod_num, int):
        macd = talib.MACD(target, fastperiod=fastperiod_num, slowperiod=slowperiod_num, signalperiod=signalperiod_num)[2] * 2
        return macd

    else:
        raise ValueError('请输入正确格式的参数, target为Series, 另外三个参数为int')



# _*_coding:utf-8_*_
# @Time: 2020/11/8 10:42
# @Author: zhang chenyong
# @File: volume_ratio.py


import time
import pandas as pd


def cal(min_volume, auc_vol_cum_volume):
    """
    今日量比因子： 今日竞价平均分钟成交量与前五日分钟平均成交量的比值
    Parameters
    ----------
    min_volume : pd.Series   分钟数据成交量  传入的是单只股票前五天（不包含今天）的分钟数据
    auc_vol_cum_volume: pd.Series or int/float  日竞价总成交量

    Returns
    -------
    Series 类型  ref0_volume_ratio

            Examples
    ______________________________________________
    0    0.060476
    Name: ref0_volume_ratio, dtype: float64
    ______________________________________________
    """

    assert isinstance(min_volume, pd.Series)
    assert isinstance(auc_vol_cum_volume, (pd.Series, int, float))
    if len(min_volume) != 1200:
        raise ValueError('传入的分钟数据不准确')

    df = pd.DataFrame()
    #  前五日分钟平均成交量
    df['five_day_mean_min_volume'] = pd.Series(min_volume.mean())
    #  今日竞价总成交量
    if isinstance(auc_vol_cum_volume, pd.Series):
        df['auc_vol_cum_volume'] = auc_vol_cum_volume.values/10
    else:
        df['auc_vol_cum_volume'] = auc_vol_cum_volume/10

    #  今日量比
    df['ref0_volume_ratio'] = df['auc_vol_cum_volume']/df['five_day_mean_min_volume']
    ref0_volume_ratio = df['ref0_volume_ratio']
    return ref0_volume_ratio


if __name__ == '__main__':
    from qytools.db_read import DBReader
    import time
    db_read = DBReader()
    start = time.time()
    df_min_volume = db_read.read_tdx_1min_data(start=20201023, end=20201023, shift=4, fields='volume,code,datetime', code=603306)
    ref0_aucvol_data = db_read.read_aucvol_data(start=20201023, end=20201023, code=603306, fields='cum_volume,code')
    auc_vol_cum_volume = ref0_aucvol_data['cum_volume']
    min_volume = df_min_volume['volume']
    print(cal(min_volume, auc_vol_cum_volume))

    end = time.time()
    print(end-start)



# _*_coding:utf-8_*_
# @Time: 2020/11/9 8:47
# @Author: zhang chenyong
# @File: aucvol2aucvol_today2yes.py

import pandas as pd


def cal(auc_vol):
    """
    今日竞价量对昨日竞价量的比值
    Parameters
    ----------
    auc_vol : Series  竞价成交量

    Returns
    -------
    Series   ref0_aucvol2aucvol_today2yes

            Examples
    ______________________________________________
    0          NaN
    1     0.258222
    2     0.132075
    3    15.857143
    4     0.076577
    5     1.764706
    Name: cum_volume, dtype: float64
    ______________________________________________

    """

    assert isinstance(auc_vol, pd.Series)
    ref0_aucvol2aucvol_today2yes = auc_vol / auc_vol.shift(1)
    return ref0_aucvol2aucvol_today2yes


if __name__ == '__main__':
    from qytools.db_read import DBReader

    db_read = DBReader()
    data = db_read.read_aucvol_data(start=20201022, end=20201029, code=603306, fields='code,date,cum_volume')
    auv_vol = data['cum_volume']
    print(cal(auv_vol))


# -*- coding:utf-8 -*-
# @Time: 2020/11/9 09:28
# @Author: jiangwenhao
# @File: ma_day_rise_ratio.py

import pandas as pd
import numpy as np


def cal(close, ma=5) -> pd.Series:
	"""
	计算个股ma日内的涨跌幅

	Parameters
	----------
	close : Series
		收盘价
	ma: int
		日期区间(ma日）默认值为5
	Returns
	-------
	# pd.Series
		个股ma日内的涨跌幅
		Examples:
			----------------------------------
			0            NaN
			1            NaN
			2            NaN
			3            NaN
			4            NaN
			...
			139    30.342499
			140    42.337165
			141    23.442547
			142    15.100671
			143    21.302251
			Name: ma_day_rise_ratio, Length: 144, dtype: float64
			----------------------------------
	"""
	if isinstance(close, pd.Series) and isinstance(ma, int):
		df_compare = pd.DataFrame({'close': close}).reset_index(drop=True)
		df_compare['last_ma_close'] = df_compare['close'].shift(ma)
		ma_day_rise_ratio = df_compare.apply(
			lambda x: (x['close']-x['last_ma_close'])/x['last_ma_close'] * 100, axis=1
		)
	else:
		raise ValueError('传入的close需为Series格式,ma需要为int格式')
	return ma_day_rise_ratio



# -*- coding:utf-8 -*-
# @Time: 2020/11/6 23:22
# @Author: wangtao
# @File: riseratio_sector.py


import pandas as pd
import numpy as np


def cal(close, timeperiod=5):
	"""
	计算区间涨幅

	Parameters
	----------
	close : Series or np.array
		收盘价
	timeperiod : int, default 5
		区间长度

	Returns
	-------
	pd.Serise
		区间涨幅
		Examples:
			----------------------------------
			0          NaN
			1          NaN
			2          NaN
			3          NaN
			4          NaN
			5    -0.004526
			...
			59    0.030500
			60    0.017884
			61    0.111111
			62    0.164908
			63    0.106291
			Name: sectorriseratio, Length: 64, dtype: float64
			----------------------------------
	"""
	
	if isinstance(close, pd.Series):
		sectorriseratio = ((close - close.shift(timeperiod)) / close.shift(timeperiod)).rename('sectorriseratio', inplace=True)
	elif isinstance(close, np.ndarray):
		sectorriseratio = (pd.Series(close) - pd.Series(close).shift(timeperiod)) / pd.Series(close).shift(timeperiod)
	else:
		raise ValueError('传入的close需为Series或np.array格式')
	return sectorriseratio


def cal_data(data, by='close', timeperiod=5):
	"""
	符合原始数据计算的区间涨幅
	为了简便运算，部分文件含有cal_data函数，可以针对日线数据库中的格式对多个股票同时操作
	
	Parameters
	----------
	data : pd.DataFrame
		原始日线数据表格式的df
		Examples:
			----------------------------------
			       code        date  close
			0    300001  2020-06-12  21.99
			1    300001  2020-06-15  21.91
			..      ...         ...    ...
			126  601216  2020-09-10   7.39
			127  601216  2020-09-11   7.32
			----------------------------------
	by ： str, default 'close'
		按照by 计算区间涨幅
	timeperiod : int, default 5
		区间长度
		
	Returns
	-------
	pd.DataFrame
		原始日线数据表格式的df，并新增了该因子列
		Examples:
			----------------------------------
					code       date  close  riseratio_sectors
			0    300001  2020-06-12  21.99                NaN
			1    300001  2020-06-15  21.91                NaN
			2    300001  2020-06-16  22.82                NaN
			3    300001  2020-06-17  22.03                NaN
			4    300001  2020-06-18  21.91                NaN
			..      ...         ...    ...                ...
			123  601216  2020-09-07   8.37          -0.105769
			124  601216  2020-09-08   8.07          -0.113187
			125  601216  2020-09-09   7.32          -0.194719
			126  601216  2020-09-10   7.39          -0.204521
			127  601216  2020-09-11   7.32          -0.188470
			----------------------------------
	"""

	if isinstance(data, pd.DataFrame) and ('code' in data.columns):
		data['riseratio_sectors'] = data.groupby('code').apply(lambda x: x[by] / x[by].shift(timeperiod) - 1).values
	else:
		raise ValueError('传入的data不是DataFrame格式或data中不含有code列')
	return data


if __name__ == '__main__':
	import qytools
	db_read = qytools.db_read.DBReader()
	df = db_read.read_ts_day_data(start=db_read.get_opendate(20200612), end=db_read.get_opendate(20200912), fields='code,date,close',code=[601216, 300001])
	df1 = df.groupby('code').get_group(300001)
	sectorriseratio_ser = cal(df1.close)
	new_data = cal_data(df)



# _*_coding:utf-8_*_
# @Time: 2020/11/8 11:17
# @Author: zhang chenyong
# @File: first_time_section.py

import pandas as pd
import numpy as np


def cal(first_time):
    """
    首次涨停时间
    Parameters
    ----------
    first_time: Series  首次涨停时间

    Returns
    -------
        int   ref1_first_time_section
    """
    # first_time 须为Series 格式
    assert isinstance(first_time, pd.Series)
    if len(first_time) != 0:
        first_time = first_time[0]
        if '09:25:00' <= first_time < '09:30:00':
            ref1_first_time_section = 0
        elif '09:30:00' <= first_time < '10:00:00':
            ref1_first_time_section = 1
        elif '10:00:00' <= first_time < '10:30:00':
            ref1_first_time_section = 2
        elif '10:30:00' <= first_time < '11:00:00':
            ref1_first_time_section = 3
        elif '11:00:00' <= first_time < '11:30:00':
            ref1_first_time_section = 4
        elif '13:00:00' <= first_time < '13:30:00':
            ref1_first_time_section = 5
        elif '13:30:00' <= first_time < '14:00:00':
            ref1_first_time_section = 6
        elif '14:00:00' <= first_time < '14:30:00':
            ref1_first_time_section = 7
        elif '14:30:00' <= first_time < '15:00:00':
            ref1_first_time_section = 8
        else:
            ref1_first_time_section = np.nan
    else:
        ref1_first_time_section = np.nan

    return ref1_first_time_section


if __name__ == '__main__':
    from qytools.db_read import DBReader
    db_read = DBReader()
    first_time = db_read.read_ts_limit_list(start=20201023, end=20201023, code=600573, fields='first_time')
    first_time = first_time['first_time']
    print(cal(first_time))



# _*_coding:utf-8_*_
# @Time: 2020/12/3 19:17
# @Author: zhang chenyong
# @File: concept_rise_ratio.py
from qytools.db_read import DBReader
import pandas as pd
import numpy as np


def cal(cod, date, code_in_con, con_in_code, db_read: DBReader):
    """
    计算股票所属主板块、副板块 以及 所属主板块涨幅、所属副板块涨幅
    Parameters
    ----------
    cod   list
    date   int
    code_in_con
    con_in_code
    db_read

    Returns  pd.DataFrame()

Examples:
________________________________________________________
       code        date ref1_first_con  ref1_first_con_riseratio
    0     6  2020-10-29           con3                 -0.060928
    -------

    """
    yesterday = date
    ts_day_data_today = db_read.read_ts_day_data(start=yesterday, end=yesterday,
                                                      fields=['code', 'date', 'pct_chg'],
                                                      stdrop=False)
    # 股票所属板块合并进日线数据
    ts_day_data_today = pd.merge(ts_day_data_today, con_in_code, on='code', how='left')
    ts_day_data_today.rename(columns={'pct_chg': 'pre_pct_chg'}, inplace=True)
    df_ref1_pct = ts_day_data_today[['code', 'pre_pct_chg']]
    #  计算板块当日的平均涨幅
    for i, icode_list in enumerate(code_in_con.code):
        ref1_pct_list = [df_ref1_pct.loc[icode, 'pre_pct_chg'] if icode in df_ref1_pct.code else 0 for icode in icode_list]
        code_in_con.loc[i, 'yes_con_pct_chg'] = np.mean(ref1_pct_list)
    code_in_con.sort_values(ascending=False, by=['yes_con_pct_chg'], ignore_index=True, inplace=True)
    #  读取所求股票数据
    ts_day_data_today_code = ts_day_data_today[ts_day_data_today['code'].isin(cod)].reset_index(drop=True)
    #  计算所求股票的因子 'ref1_second_con' 'ref1_second_con_riseratio' 'ref1_second_con'  'ref1_second_con_riseratio'
    for i, con_list in enumerate(ts_day_data_today_code.concept):
        # 寻找主板块和副板块
        if isinstance(con_list, list):
            # 主板块:昨日涨幅最大的板块
            if len(con_list) > 0:
                try:
                    ts_day_data_today_code.loc[i, 'ref1_first_con'] = \
                        code_in_con[code_in_con.concept.isin(con_list)].concept.iloc[0]
                except:
                    pass
                try:
                    ts_day_data_today_code.loc[i, 'ref1_first_con_riseratio'] = \
                        code_in_con[code_in_con.concept.isin(con_list)].yes_con_pct_chg.iloc[0]
                except:
                    pass
            # 股票涉及两个以上板块时，寻找副板块
            if len(con_list) > 1:
                try:
                    ts_day_data_today_code.loc[i, 'ref1_second_con'] = \
                        code_in_con[code_in_con.concept.isin(con_list)].concept.iloc[1]
                except:
                    pass
                try:
                    ts_day_data_today_code.loc[i, 'ref1_second_con_riseratio'] = \
                        code_in_con[code_in_con.concept.isin(con_list)].yes_con_pct_chg.iloc[1]
                except:
                    pass
    df_concept = ts_day_data_today_code.drop(['concept', 'pre_pct_chg'], axis=1)
    return df_concept


if __name__ == '__main__':
    cod = [78]
    yesterday = 20201029
    db_read = DBReader()
    _, code_in_con = db_read.read_concept_data(dfconcept=True, dropstandard=200)  # 板块包股票信息
    _, con_in_code = db_read.read_concept_data(dfcode=True, dropstandard=200)  # 股票包板块信息
    cal(code_in_con=code_in_con, con_in_code=con_in_code, cod=cod, date=yesterday, db_read=db_read)


# -*- coding:utf-8 -*-
# @Time: 2020/11/8 16:44
# @Author: jiangwenhao
# @File: riseoverstock_bias_ma.py

import pandas as pd
import numpy as np


def cal(close, up_limit, ma=5) -> pd.Series:
	"""
	计算涨停表现偏差度

	Parameters
	----------
	close : Series
		收盘价
	up_limit : Series
		涨停价（index应与close对应）
	ma: int
		日期区间(ma日）
	Returns
	-------
	pd.Series or np.array
		涨停表现偏差度
		Examples:
			----------------------------------
			0          NaN
			1          NaN
			2          NaN
			3          NaN
			...
			50         NaN
			51         NaN
			52    1.194458
			Name: riseoverstock_bias_ma, Length: 53, dtype: float64
			----------------------------------
	"""
	if isinstance(close, pd.Series) and isinstance(up_limit, pd.Series) and isinstance(ma, int):
		df_compare = pd.DataFrame({'close': close, 'up_limit': up_limit}).reset_index(drop=True)
		df_compare['is_up_limit'] = 0
		df_compare.loc[round(df_compare['close'], 2) == round(df_compare['up_limit'], 2), 'is_up_limit'] = 1
		df_compare['rise_ratio_cum_prod'] = ((df_compare['close'] - df_compare['close'].shift(1))/df_compare['close'].shift(1)+1).cumprod()
		df_compare['rise_ratio_ma'] = df_compare['rise_ratio_cum_prod'].rolling(window=ma).mean().shift(1)
		riseoverstock_bias_ma = df_compare.apply(
			lambda x: x['rise_ratio_cum_prod']/x['rise_ratio_ma'] if x['is_up_limit'] == 1 else np.nan, axis=1
		)
	else:
		raise ValueError('传入的close和up_limit需为Series格式,ma需要为int格式')
	return riseoverstock_bias_ma



# -*- coding:utf-8 -*-
# @Time: 2020/11/6 23:22
# @Author: wangtao
# @File: riseratio_sector.py


import pandas as pd


def cal(close, up_limit, timeperiod=5):
	"""
	计算区间涨停股数(talib形式)
	注：默认传入的Series日期是升序排列（index越大，日期越新）
	
	Parameters
	----------
	close : Series
		收盘价
	up_limit : Series
		涨停价
	timeperiod : int, default 5
		区间长度
		
	Returns
	-------
	pd.Series
		区间涨停股数
		Examples:
			----------------------------------
			0      NaN
			1      NaN
			2      NaN
			3      NaN
			4      1.0
			...
			351    0.0
			352    0.0
			353    0.0
			354    0.0
			355    0.0
			Name: riseovernum_sector, Length: 356, dtype: float64
			----------------------------------
		
	"""
	
	if isinstance(close, pd.Series) and isinstance(up_limit, pd.Series):
		df_compare = pd.DataFrame({'close': close, 'up_limit': up_limit})
		df_compare['is_uplimit'] = 0
		df_compare.loc[round(df_compare['close'], 2) == round(df_compare['up_limit'], 2), 'is_uplimit'] = 1
		riseovernum_sector = df_compare['is_uplimit'].rolling(timeperiod).sum().rename('riseovernum_sector', inplace=True)
	else:
		raise ValueError('传入的close和up_limit需为Series或np.array格式')
	return riseovernum_sector


def cal_data(data, timeperiod=5):
	"""
	计算区间涨停股数(可对含多个code的原始日线表进行处理)
	注：默认传入的Series日期是升序排列（index越大，日期越新）
	
	Parameters
	----------
	data : pd.DataFrame
		日线数据表
		Examples:
			----------------------------------
			       code        date  close  up_limit
			0    300001  2020-06-12  21.99     23.82
			1    300001  2020-06-15  21.91     24.19
			2    300001  2020-06-16  22.82     24.10
			3    300001  2020-06-17  22.03     25.10
			4    300001  2020-06-18  21.91     24.23
			..      ...         ...    ...       ...
			107  601216  2020-08-26   9.80      9.80
			108  601216  2020-08-27   9.31     10.78
			109  601216  2020-08-28   9.05     10.24
			110  601216  2020-08-31   9.36      9.96
			111  601216  2020-09-01   9.10     10.30
			----------------------------------
	timeperiod : int, default 5
		区间长度
		
	Returns
	-------
	pd.DataFrame
		传入的data新增一列区间涨停股数
		Examples:
			--------------------------------------------------------------------
			       code        date  close  up_limit  riseovernum_sector
			0    300001  2020-06-12  21.99     23.82                 NaN
			1    300001  2020-06-15  21.91     24.19                 NaN
			2    300001  2020-06-16  22.82     24.10                 NaN
			3    300001  2020-06-17  22.03     25.10                 NaN
			4    300001  2020-06-18  21.91     24.23                 0.0
			..      ...         ...    ...       ...                 ...
			107  601216  2020-08-26   9.80      9.80                 1.0
			108  601216  2020-08-27   9.31     10.78                 1.0
			109  601216  2020-08-28   9.05     10.24                 1.0
			110  601216  2020-08-31   9.36      9.96                 1.0
			111  601216  2020-09-01   9.10     10.30                 1.0
			--------------------------------------------------------------------
	"""
	
	if isinstance(data, pd.DataFrame) and ('code' in data.columns):
		new_data = data.copy()
		new_data['is_uplimit'] = 0
		new_data.loc[round(new_data['close'], 2) == round(new_data['up_limit'], 2), 'is_uplimit'] = 1
		data['riseovernum_sector'] = new_data.groupby('code')['is_uplimit'].rolling(timeperiod).sum().values
	else:
		raise ValueError('传入的data不是DataFrame格式或data中不含有code列')
	return data


if __name__ == '__main__':
	import qytools
	db_read = qytools.db_read.DBReader()
	df = db_read.read_ts_day_data(start=db_read.get_opendate(20200612), end=db_read.get_opendate(20200901), fields='code,date,close,up_limit', code=[601216,300001])
	df1 = df.groupby('code').get_group(601216)
	riseovernum_sector_series = cal(df1.close, df1.up_limit)
	new_data = cal_data(df)



from . import aucvol2aucvol_today2yes
from . import aucvol2maxvol_today2yes
from . import beta
from . import cross_n_min_amount
from . import cross_n_min_vol
from . import first_time_section
from . import firstcrosstime
from . import is_price_peak
from . import last_time_section
from . import ln_hl
from . import lowratio
from . import ma_day_rise_ratio
from . import macd
from . import macd_adj
from . import market_pearson_adj
from . import max_vol
from . import min_bomb_time
from . import n_cross_time
from . import net_inflow_rate_volume
from . import net_inflow_rate_volume_l
from . import o2c_today2yes
from . import rise_speed_n_min
from . import riseover_amount
from . import riseover_continue
from . import riseovernum_nday
from . import riseovernum_sector
from . import riseoverstock_bias_ma
from . import riseratio_sector
from . import s_li_largebuyrate
from . import s_risk_glvarianceratio
from . import s_tech_cyf
from . import s_tech_dhilo
from . import s_tech_rank
from . import s_tech_revs
from . import s_tech_rvi
from . import s_tech_volumn
from . import turn_over_ratio2MA5
from . import turnover_sector
from . import value_diff_institute
from . import volume_ratio


# _*_coding:utf-8_*_
# @Time: 2020/11/9 9:56
# @Author: zhang chenyong
# @File: aucvol2maxvol_today2yes.py


import pandas as pd


def cal(auc_vol_cum_volume, minute_vol):
    """
    今日竞价平均分钟量与昨日最大分钟量的比值
    Parameters
    ----------
    auc_vol_cum_volume : Series  今日竞价量  or  int/float
    minute_vol : Series  昨日分钟数据

    Returns
    -------
    Series  ref0_aucvol2maxvol_today2yes
        Examples
            ______________________________________________
            0    0.01403
            Name: ref0_aucvol2maxvol_today2yes, dtype: float64
            ______________________________________________
    """

    assert isinstance(auc_vol_cum_volume, (pd.Series, int, float))
    assert isinstance(minute_vol, pd.Series)
    df = pd.DataFrame()
    #  今日竞价总成交量
    if isinstance(auc_vol_cum_volume, pd.Series):
        df['auc_vol_cum_volume'] = auc_vol_cum_volume.values / 10
    else:
        df['auc_vol_cum_volume'] = pd.Series(auc_vol_cum_volume / 10)
    #  昨日分钟最大成交量
    df['max_min_volume'] = pd.Series(minute_vol.max())
    #  今日竞价平均分钟量对昨日最大分钟量
    df['ref0_aucvol2maxvol_today2yes'] = df['auc_vol_cum_volume']/df['max_min_volume']
    ref0_aucvol2maxvol_today2yes = df['ref0_aucvol2maxvol_today2yes']
    return ref0_aucvol2maxvol_today2yes


if __name__ == '__main__':
    from qytools.db_read import DBReader

    db_read = DBReader()
    df_min_volume = db_read.read_tdx_1min_data(start=20201023, end=20201023, fields='volume,code,datetime', code=603306)
    ref0_aucvol_data = db_read.read_aucvol_data(start=20201023, end=20201023, code=603306, fields='cum_volume,code')
    auc_vol_cum_volume = ref0_aucvol_data['cum_volume']
    minute_vol = df_min_volume['volume']
    print(cal(auc_vol_cum_volume, minute_vol))



# -*- coding:utf-8 -*-
# @Time: 2020/11/10 13:30
# @Author: caiyucheng
# @File: cross_n_min_amount.py

import pandas as pd
import numpy as np


def cal(vol, close, n):
    """
    当前交易量相比过去一段时期平均交易量 * 过去一段时期的累积收益率乘积
    Parameters
    ----------
    vol: pd.Series
         成交量
    close: pd.Series
           收盘价
    Returns
    -------
        pd.Series
        Examples
        --------
        0        NaN
        1        NaN
        2        NaN
        3        NaN
        4        NaN
        5        NaN
        6        NaN
        7        NaN
        8        NaN
        9        NaN
        10       NaN
        11       NaN
        12       NaN
        13       NaN
        14       NaN
        15       NaN
        16       NaN
        17       NaN
        18       NaN
        19       NaN
        20   -0.1182
        21   -0.1981
        22   -0.1737
        23   -0.1352
        24   -0.1104
        25   -0.1401
        26   -0.1035
        27   -0.0866
        28   -0.0682
        29   -0.0589
        Name: x, dtype: float64
        -----------------------
    """
    if isinstance(vol, pd.Series) and isinstance(close, pd.Series) and isinstance(n, int):
        cum_ret = close / close.shift(n) - 1
        s_tech_vol = (vol / vol.rolling(n).mean()) * cum_ret
        return s_tech_vol
    else:
        raise ValueError('请输入正确格式的参数, close, vol为Series, n为int')


# _*_coding:utf-8_*_
# @Time: 2020/11/8 21:19
# @Author: zhang chenyong
# @File: riseover_amount.py


import pandas as pd


def cal(low, amount, up_limit):
    """
    板上成交额因子： 分钟最低价等于涨停价的分钟成交额的总量

    Parameters
    ----------
    low：pd.Series    分钟最低价
        Examples:

                0      8.23
                1      8.24
                2      8.28
                3      8.27
                4      8.44
                       ...
                235    9.03
                236    9.03
                237    9.03
                238    9.03
                239    9.03
                Name: low, Length: 240, dtype: float64
            ______________________________
    amount： pd.Series     分钟成交额

        Examples

                0      13567572.00
                1       2488528.00
                2       3892867.00
                3       4631776.00
                4       5022798.00
                          ...
                235      200466.00
                236       59598.00
                237           0.00
                238           0.00
                239      701631.06
                Name: amount, Length: 240, dtype: float64
            ______________________________
    up_limit： Series  当日股票涨停价

        Examples:

                0    9.03
                Name: up_limit, dtype: float64
            ______________________________
            Returns
    -------
        int类型   ref1_riseover_amount
    """

    # 判断传入分钟最高价和分钟收盘价为Series格式
    assert isinstance(low, pd.Series)
    assert isinstance(amount, pd.Series)
    # 传入涨停价必须是浮点数或整数格式
    assert isinstance(up_limit, pd.Series)
    # 取出分钟最高价和收盘价
    df = pd.DataFrame()
    df['low'] = low
    df['amount'] = amount
    # 涨停价
    df['up_limit'] = up_limit[0]
    # 涨停分钟成交额
    df['ref1_riseover_amount'] = [df.loc[i, 'amount'] if df.loc[i, 'low'] == df.loc[i, 'up_limit']
                                    else 0 for i in df.index]
    # 板上成交额
    ref1_riseover_amount = df['ref1_riseover_amount'].sum()

    return ref1_riseover_amount


if __name__ == '__main__':
    from qytools.db_read import DBReader
    db_read = DBReader()
    data = db_read.read_tdx_1min_data(start=20201023, end=20201023, fields=['low', 'amount'], code=600573)
    up_limit = db_read.read_ts_day_data(start=20201023, end=20201023, fields='up_limit', code=600573)
    low = data['low']
    amount = data['amount']
    up_limit = up_limit['up_limit']
    print(cal(low, amount, up_limit))


# -*- coding:utf-8 -*-
# @Time: 2020/11/7 14:45
# @Author: wangtao
# @File: riseover_continue.py

import pandas as pd


def cal(close: pd.Series, up_limit: pd.Series):
	"""
	计算连板数

	Parameters
	----------
	close : Series
		收盘价
	up_limit : Series
		涨停价（index应与close对应）
		
	Returns
	-------
	pd.Series
		连板数
		Examples:
			----------------------------------
			0      0.0
			1      0.0
			...
			176    0.0
			177    0.0
			Name: riseovernum_continue, Length: 178, dtype: float64
			----------------------------------
	"""
	
	if isinstance(close, pd.Series) and isinstance(up_limit, pd.Series):
		df_compare = pd.DataFrame({'close': close, 'up_limit': up_limit}).reset_index(drop=True)
		df_compare['is_uplimit'] = 0
		df_compare.loc[round(df_compare['close'], 2) == round(df_compare['up_limit'], 2), 'is_uplimit'] = 1
		riseovernum_continue = df_compare.rolling(window=20, min_periods=1)['is_uplimit'].apply(
			lambda x: x.index.max() - x[x == 0].index.max()).rename('riseovernum_continue', inplace=True)
	else:
		raise ValueError('传入的close需为Series格式')
	return riseovernum_continue


if __name__ == '__main__':
	import qytools
	db_read = qytools.db_read.DBReader()
	df = db_read.read_ts_day_data(start=20200612, end=20200912, fields='code,date,close,up_limit', code=[601216])
	cal(df.close, df.up_limit)



# -*- coding:utf-8 -*-
# @Time: 2020/11/10 19:05
# @Author: caiyucheng
# @File: cross_n_min_amount.py

import pandas as pd
import numpy as np


def cal(buy_lg_amount, buy_elg_amount, amount):
    """
    大单买入成交金额 / 总成交金额
    Parameters
    ----------
    buy_lg_amount: pd.Series
                   大单买入额
    buy_elg_amount: pd.Series
                特大单买入额
    amount: pd.Series
            成交金额
    Returns
    -------
    pd.Series
        Examples
        --------
        0    0.0704
        1    0.0703
        2    0.0568
        3    0.0428
        4    0.0487
        5    0.0611
        6    0.0479
        7    0.0662
        8    0.0605
        9    0.0477
        Name: x, dtype: float64
        -----------------------
    """
    if isinstance(buy_lg_amount, pd.Series) and isinstance(buy_elg_amount, pd.Series) and isinstance(amount, pd.Series):
        return (buy_elg_amount + buy_lg_amount) / amount

    else:
        raise ValueError('请输入格式正确的参数, buy_lg_amount, buy_elg_amount, amount均为Series')


# -*- coding:utf-8 -*-
# @Time: 2020/11/10 19:05
# @Author: caiyucheng
# @File: cross_n_min_amount.py

import pandas as pd
import numpy as np


def cal(buy_lg_vol,  buy_elg_vol, sell_lg_vol, sell_elg_vol, vol):
    """
    大单流入率 (buy_lg_vol + buy_elg_vol - sell_lg_vol - sell_elg_vol) / vol
    Parameters
    ----------
    buy_lg_vol: pd.Series
                大单买入量
    buy_elg_vol: pd.Series
                特大单买入量
    sell_lg_vol: pd.Series
                大单卖出量
    sell_elg_vol: pd.Series
                 特大单卖出量
    vol: pd.Series
         成交量
    Returns
    -------
    pd.Series
        Examples
        --------
        0    0.0544
        1    0.1782
        2   -0.0486
        3   -0.1316
        4   -0.1852
        5   -0.0694
        6   -0.0621
        7    0.0774
        8   -0.0936
        9   -0.1230
        Name: x, dtype: float64
        -----------------------
    """
    if isinstance(buy_lg_vol, pd.Series) and isinstance(buy_elg_vol, pd.Series) and isinstance(sell_lg_vol, pd.Series) and isinstance(sell_elg_vol, pd.Series) and isinstance(vol, pd.Series):
        return (buy_lg_vol + buy_elg_vol - sell_lg_vol - sell_elg_vol) / vol
    else:
        raise ValueError('请输入正确格式的参数buy_lg_vol,  buy_elg_vol, sell_lg_vol, sell_elg_vol, vol均为Series')


# -*- coding:utf-8 -*-
# @Time: 2020/11/10 13:30
# @Author: caiyucheng
# @File: cross_n_min_amount.py

import pandas as pd
import numpy as np
import talib


def cal(turnover_rate, n=20):
    """
    市场能量指标 = 100 - 100/(1 + EMA(turnover_rate))  默认换手率均线参数为20个周期
    Parameters
    ----------
    turnover_rate: Series
                   股票换手率
    n: int
       换手率均线指标
    Returns
    -------
    pd.Series
        Examples
        --------
            0         NaN
            1         NaN
            2         NaN
            3         NaN
            4         NaN
            5         NaN
            6         NaN
            7         NaN
            8         NaN
            9     33.4137
            10    33.6386
            11    32.0404
            12    31.3022
            13    31.3567
            14    30.6110
            15    31.6963
            16    37.1603
            17    39.1611
            18    39.9897
            19    39.6255
            Name: x, dtype: float64
            -----------------------
    """
    if isinstance(turnover_rate, pd.Series) and isinstance(n, int):
        EMA_ = pd.Series(talib.EMA((turnover_rate), timeperiod=n), name='EMA')
        CYF = 100 - 100 / (1 + EMA_)
        return CYF
    else:
        raise ValueError('请输入正确格式的参数, vol, flo_share为')



# -*- coding:utf-8 -*-
# @Time: 2020/11/8 16:44
# @Author: jiangwenhao
# @File: o2c_today2yes.py

import pandas as pd
import numpy as np


def cal(open, close) -> pd.Series:
	"""
	计算今日开盘涨幅

	Parameters
	----------
	open : Series
		开盘价
	close : Series
		收盘价

	Returns
	-------
	pd.Series or np.array
		今日开盘涨幅
		Examples:
			----------------------------------
			0           NaN
			1      0.671592
			2     -1.623816
			3     -3.349616
			4      0.652647
			...
			120   -1.320293
			121   -1.414353
			122    3.736264
			123   -0.649351
			124    1.185345
			Name: o2c_today2yes, Length: 125, dtype: float64
			----------------------------------
	"""
	if isinstance(open, pd.Series) and isinstance(close, pd.Series):
		df_compare = pd.DataFrame({'open': open, 'close': close}).reset_index(drop=True)
		df_compare['last_close'] = df_compare['close'].shift(1)
		o2c_today2yes = df_compare.apply(lambda x: ((x['open']-x['last_close'])/x['last_close'])*100, axis=1)
	else:
		raise ValueError('传入的open和close需为Series格式')
	return o2c_today2yes



# -*- coding:utf-8 -*-
# @Time: 2020/11/10 20:49
# @Author: caiyucheng
# @File: cross_n_min_amount.py

import pandas as pd
import numpy as np

def cal(high, low, n):
    """
    波幅中位数 n周期内 最高价 - 最低价的中位数
    Parameters
    ----------
    high: Series
          股票最高价
    low: Series
         股票最低价
    n: int
       计算区间
    Returns
    pd.Series
    Examples
    -------


    """
    if isinstance(high, pd.Series) and isinstance(low, pd.Series) and isinstance(n, int):
        s_tech_dhilo = (high - low).rolling(window=n).median()
        return s_tech_dhilo

    else:
        raise ValueError('请输入正确格式的参数, high, low为Series, n为int')


# -*- coding:utf-8 -*-
# @Time: 2020/11/10 13:30
# @Author: caiyucheng
# @File: cross_n_min_amount.py

import pandas as pd
import numpy as np


def cal(stock_close, index_close, n=20):
    """
    计算周期为n区间内的股票beta值
    Parameters
    ----------
    stock_close: Series
                 股票收盘价
    index_close: Series
                 指数收盘价
    n: int
       计算区间

    Returns
    ----------
    pd.Series
        个股n日beta
        Examples
    ----------
            0        NaN
            1        NaN
            2        NaN
            3        NaN
            4        NaN
            5        NaN
            6     0.0476
            7     0.5830
            8     0.5028
            9     0.5081
            10    1.3604
            11    0.8478
            12    0.8245
            13    0.2463
            14    0.4849
            Name: x, dtype: float64
            -----------------------
    """
    if isinstance(stock_close, pd.Series) and isinstance(index_close, pd.Series) & isinstance(n, int):
        df = pd.concat([stock_close, index_close], axis=1).reset_index(drop=True).copy()
        df.columns = ['x', 'y']
        df[df.columns[0]], df[df.columns[1]] = df[df.columns[0]].pct_change(1), df[df.columns[1]].pct_change(1)
        list_index = df.index.tolist()
        df['beta'] = [__get_beta(index_=index, df_=df.copy(), n_=n) for index in list_index]
    else:
        raise ValueError('传入的stock_close, index_close需为Series格式,n需为int格式')
    return df['beta']


def __get_beta(index_, df_, n_):
    if index_ - n_ < 0:
        return np.nan
    else:
        a = index_ - n_    # 不写ab的话后面全是nan 不知道为什么
        b = index_
        stk_ret = df_['x'][a: b]
        index_ret = df_['y'][a: b]
        beta = np.cov(stk_ret, index_ret)[0][1] / np.var(index_ret)
        return beta



# _*_coding:utf-8_*_
# @Time: 2020/11/8 9:16
# @Author: zhang chenyong
# @File: min_bomb_time.py
import pandas as pd


def cal(high, close, up_limit):
    """
    分钟炸板次数因子

    Parameters
    ----------
    high：Series    分钟最高价
        Examples :
            _________________________
                    0      8.39
                    1      8.35
                    2      8.32
                    3      8.46
                    4      8.51
                           ...
                    235    9.03
                    236    9.03
                    237    9.03
                    238    9.03
                    239    9.03
                    Name: high, Length: 240, dtype: float64
            ___________________________________

    close： Series     分钟收盘价
        Examples :
            _________________________
                    0      8.24
                    1      8.33
                    2      8.28
                    3      8.46
                    4      8.44
                           ...
                    235    9.03
                    236    9.03
                    237    9.03
                    238    9.03
                    239    9.03
                    Name: close, Length: 240, dtype: float64
            ___________________________________

    up_limit： Series  当天股票涨停价

    Returns
    -------
        int类型   分钟炸板次数
    """

    # 判断传入分钟最高价和分钟收盘价为Series格式
    assert isinstance(high, pd.Series)
    assert isinstance(close, pd.Series)
    # 传入涨停价 须是Series格式
    assert isinstance(up_limit, pd.Series)
    # 取出分钟最高价和收盘价
    df = pd.DataFrame()
    df['high'] = high
    df['close'] = close
    # 涨停价
    df['up_limit'] = up_limit[0]
    # 分钟是否炸板
    df['min_bomb'] = [1 if (df.loc[i, 'high'] == df.loc[i, 'up_limit']) & (df.loc[i, 'close'] != df.loc[i, 'up_limit'])
                      else 0 for i in df.index]
    # 总炸板次数
    ref1_min_bomb_time = df['min_bomb'].sum()

    return ref1_min_bomb_time


if __name__ == '__main__':
    from qytools.db_read import DBReader
    db_read = DBReader()
    data = db_read.read_tdx_1min_data(start=20201023, end=20201023, fields=['high', 'close'], code=600573)
    up_limit = db_read.read_ts_day_data(start=20201023, end=20201023, fields='up_limit', code=600573)
    high = data['high']
    close = data['close']
    up_limit = up_limit['up_limit']
    print(cal(high, close, up_limit))



# -*- coding:utf-8 -*-
# @Time: 2020/11/8 16:44
# @Author: jiangwenhao
# @File: low_ratio.py

import pandas as pd
import numpy as np


def cal(close, low):
	"""
	计算个股今日最低价涨跌幅

	Parameters
	----------
	close : Series
		收盘价
	low : Series
		最低价

	Returns
	-------
	pd.Series or np.array
		个股今日最低价涨跌幅
		Examples:
			----------------------------------
			0           NaN
			1     -4.378981
			2      0.082372
			3      0.803213
			4     -1.489028
			...
			139   -2.331190
			140   -1.554404
			141   -9.993271
			142   -3.439252
			143   -3.024781
			Name: low_ratio, Length: 144, dtype: float64
			----------------------------------
	"""
	if isinstance(close, pd.Series) and isinstance(low, pd.Series):
		df_compare = pd.DataFrame({'close': close, 'low': low}).reset_index(drop=True)
		df_compare['last_day_close'] = df_compare['close'].shift(1)
		low_ratio = df_compare.apply(
			lambda x: ((x['low'] - x['last_day_close'])/x['last_day_close']) * 100, axis=1
		)
	else:
		raise ValueError('传入的close和low需为Series格式')
	return low_ratio



# -*- coding:utf-8 -*-
# @Time: 2020/11/10 19:05
# @Author: caiyucheng
# @File: cross_n_min_amount.py

import pandas as pd
import numpy as np


def cal(data, n):
    """
    1 - 过去n个周期的累积收益率排名 / 股票总数
    Parameters
    ----------
    data: pd.DataFrame
          包含close的dataframe
    n: int
       计算区间
    Returns
    -------
    pd.DataFrame
    返回的data中较原data新增一列s_tech_rank表示股票n个周期累积收益率的倒数排名(取倒数是因为正向因子)
        Examples
        --------
            code        date  adj_factor  ...         vol  volume_ratio  s_tech_rank
            0            1  2020-01-02     109.169  ...  1.5302e+06          2.18          NaN
            1            1  2020-01-03     109.169  ...  1.1162e+06          1.21          NaN
            2            1  2020-01-06     109.169  ...  8.6208e+05          0.80          NaN
            3            1  2020-01-07     109.169  ...  7.2861e+05          0.70          NaN
            4            1  2020-01-08     109.169  ...  8.4782e+05          0.86          NaN
            ...         ...         ...  ...         ...           ...          ...
            127530  603999  2020-02-24       2.444  ...  1.5450e+05          1.06       0.6571
            127531  603999  2020-02-25       2.444  ...  1.7454e+05          1.16       0.4700
            127532  603999  2020-02-26       2.444  ...  1.1746e+05          0.75       0.3190
            127533  603999  2020-02-27       2.444  ...  8.0846e+04          0.52       0.1376
            127534  603999  2020-02-28       2.444  ...  1.3784e+05          0.98       0.0759
            [127535 rows x 42 columns]
            ----------------------------------------------------------------------------------
    """
    if isinstance(data, pd.DataFrame):
        if 'close' and 'code' and 'date' not in data.columns.tolist():
            raise Exception('输入的data中必须有code、close、date')
        data_test = data[['date', 'code', 'close']].copy().pivot_table(index='date', columns='code', values='close').pct_change(periods=n).T.rank(method='first')
        # data_test = data_test.pct_change(periods=n).T.rank(method='first')
        len_data, col_, index_ = len(data_test), data_test.columns.tolist(), data_test.index.tolist()
        data_test = pd.DataFrame(np.array(data_test) / len_data, index=index_, columns=col_).stack().reset_index()
        data_test.columns = ['code', 'date', 's_tech_rank']
        data = pd.merge(data, data_test, how='left', on=['code', 'date'])
        return data

    else:
        raise ValueError('请输入格式正确的参数')



# -*- coding:utf-8 -*-
# @Time: 2020/11/7 10:12
# @Author: wangtao
# @File: turnover_sector.py

import pandas as pd
import numpy as np


def cal(volume: pd.Series, flo_share: pd.Series, timeperiod=5) -> pd.Series:
	"""
	计算区间换手率
	注：默认传入的Series日期是升序排列（index越大，日期越新）

	Parameters
	----------
	volume : Series
		换手率
	flo_share : Series
		流通股本（数）
	timeperiod : int, default 5
		区间长度

	Returns
	-------
	pd.Serise
		区间换手率
		Examples:
			----------------------------------
			0           NaN
			1           NaN
			2           NaN
			3           NaN
			4      1.648735
			...
			59    24.722192
			60    24.622175
			61    24.275008
			62    22.061999
			63    21.538738
			Length: 64, dtype: float64
			----------------------------------
	"""

	if isinstance(volume, pd.Series) and isinstance(flo_share, pd.Series):
		sectorturnover = volume.rolling(timeperiod).sum() / flo_share
	else:
		raise ValueError('传入的volume和flo_share需为Series')
	return sectorturnover


if __name__ == '__main__':
	import qytools

	db_read = qytools.db_read.DBReader()
	df = db_read.read_ts_day_data(
		start=db_read.get_opendate(20200612),
		end=db_read.get_opendate(20200912),
		fields='code,date,vol,flo_share,close,turnover_rate',
		code=[601216]
	)
	sectorturnover_ser = cal(df.vol, df.flo_share)



# _*_coding:utf-8_*_
# @Time: 2020/11/8 9:16
# @Author: chen shigang
# @File: macd.py


import numpy as np
from numpy import array
import pandas as pd
import talib as ta
from sklearn.linear_model import LinearRegression
from qytools.db_read import DBReader


def cal(data_series, fast_period=12, slow_period=26, signal_period=9):
	"""
	series的macd三根柱子的回归斜率和信号
	Parameters
	----------

	data_series:series 计算macd的目标series
	fast_period:int 快周期线
	slow_period:int 慢周期线
	signal_period:int 快慢线的n周期平均数

	Returns
	-------

    macd_hist_reg: float macd回归斜率
    macd_signal_value: float macd信号值
	"""
	if isinstance(data_series, pd.Series):
		macd, macd_signal, macd_hist = ta.MACD(
			np.array(data_series), fast_period, slow_period, signal_period)
	elif isinstance(data_series, np.ndarray):
		macd, macd_signal, macd_hist = ta.MACD(
			data_series, fast_period, slow_period, signal_period)
	else:
		raise ValueError('传入的data_series需为Series或np.array格式')
	macd_hist_reg = pd.Series(macd_hist).rolling(3).apply(lambda x: cal_regression(x))
	# MACD信号
	condition = [(macd_hist < 0) & (macd_hist_reg > 0.001),
	             (macd_hist > 0) & (macd_hist_reg > 0.001),
	             (macd_hist > 0) & (abs(macd_hist_reg) <= 0.001),
	             (macd_hist > 0) & (macd_hist_reg < -0.001),
	             (macd_hist < 0) & (macd_hist_reg < -0.001),
	             (macd_hist < 0) & (abs(macd_hist_reg) <= 0.001)]
	macd_signal_symbol = [1, 2, 3, -1, -2, -3]
	macd_signal_value = np.select(condition, macd_signal_symbol)
	return macd_hist_reg, macd_signal_value


def cal_regression(macdhist):
	"""
    计算回归斜率
    Parameters
    ----------
    MACDhist ：float macd柱子的数值
    Returns
    -------

    reg_linear.coef_[0]   ：float MACDhist的回归斜率
    """
	reg_linear = LinearRegression().fit(
		array([1, 2, 3]).reshape(-1, 1), macdhist)
	return reg_linear.coef_[0]


if __name__ == '__main__':
	db_read = DBReader()
	data = db_read.read_ts_day_data(start=20190101, end=20200501, code=1)
	data['MACDhist_reg'], data['MACD_signal'] = cal(data.close)
	a = 1


import numpy as np
from numpy import array
import pandas as pd
from qytools.db_read import DBReader


def cal(high, timeperiod=20):
	"""
	最高价是否创前20日新高
	Parameters
	----------
	high:float 最高价series
	timeperiod:int 算前n日的新高

	Returns
	-------
	pd.Series
	is_price_peak:float
	Examples:
			----------------------------------
			0     NaN
			1     NaN
			2     NaN
			3     NaN
			4     NaN
			5     NaN
			6     NaN
			7     NaN
			8     NaN
			9     NaN
			10    NaN
			11    NaN
			12    NaN
			13    NaN
			14    NaN
			15    NaN
			16    NaN
			17    NaN
			18    NaN
			19    0.0
			20    1.0
			21    1.0
			22    1.0
			23    0.0
			24    1.0
			Name: high, dtype: float64
			----------------------------------
	"""

	if isinstance(high, pd.Series):
		to_yes_price_peak = high.rolling(timeperiod).max()
		is_price_peak = high-to_yes_price_peak
		is_price_peak[is_price_peak == 0] = 1
		is_price_peak[is_price_peak<0] = 0
	else:
		raise ValueError('传入的high需为Series格式')
	return is_price_peak


def judge_peak(yes_high, before_20high):
	"""
    判断昨日high是否是新高
	Parameters
	----------
	yes_high:series 昨日最高价
	before_20high: 前20日的最高价

	Returns
	-------
    1 or 0
	"""
	if yes_high >= before_20high:
		return 1
	else:
		return 0


if __name__ == '__main__':
	db_read = DBReader()
	data = db_read.read_ts_day_data(start=20190101, end=20200501, code=1)
	data['is_price_peak'] = cal(data.high)
	a = 1



# _*_coding:utf-8_*_
# @Time: 2020/11/8 21:35
# @Author: zhang chenyong
# @File: turn_over_ratio2MA5.py


import pandas as pd


def cal(turn_over_rate, MA=5):
    """
     计算换手率五日偏差： 换手率与五日换手率均值的比值
    Parameters
    ----------
    turn_over_rate : Series  换手率 (传入有时间顺序的换手率序列)
        Examples

            0     1.3995
            1     1.6192
            2     1.0316
            3     0.6638
            4     0.8834
            5     1.0744
            6     1.3549
            7     1.2594
            8     2.3724
            9     2.3801
            10    1.6741
            Name: turnover_rate, dtype: float64
        ______________________________

    MA : int  几日偏差

    Returns
    -------
    Series  ref1_turn_over_ratio2MA5
        Examples
            0          NaN
            1          NaN
            2          NaN
            3          NaN
            4     0.789102
            5     1.018891
            6     1.352709
            7     1.202659
            8     1.708114
            9     1.409811
            10    0.925848
            Name: ref1_turn_over_ratio2MA5, dtype: float64
        ______________________________

    """

    assert isinstance(turn_over_rate, pd.Series)
    assert isinstance(MA, int)

    if len(turn_over_rate) < MA + 1:
        raise ValueError('传入的换手率数据不够')
    df = pd.DataFrame()
    df['turn_over_rate'] = turn_over_rate
    df['turn_over_rate_MA'] = turn_over_rate.rolling(MA).mean()
    df['ref1_turn_over_ratio2MA5'] = df['turn_over_rate'] / df['turn_over_rate_MA']
    ref1_turn_over_ratio2MA5 = df['ref1_turn_over_ratio2MA5']
    return ref1_turn_over_ratio2MA5


if __name__ == '__main__':
    from qytools.db_read import DBReader

    db_read = DBReader()
    data = db_read.read_ts_day_data(
                                    start=20201022,
                                    end=20201106,
                                    code=603306,
                                    fields='code,date,turnover_rate'
                                    )
    turn_over_rate = data['turnover_rate']
    cal(turn_over_rate)





# -*- coding:utf-8 -*-
# @Time: 2020/11/10 13:30
# @Author: caiyucheng
# @File: cross_n_min_amount.py

import pandas as pd
import numpy as np


def cal(close, n, m=1):
    """
    计算n区间内的m区间的收益损失方差比
    Parameters
    ----------
    close: Series
           股票收盘价
    n: int
       计算收益损失方差比的区间
    m: int
       计算收益区间
    Returns
    -------
    pd.Series
        个股n个区间内的收益损失方差比
        Examples
    ----------
            0       NaN
            1       NaN
            2       NaN
            3       NaN
            4       NaN
            5    0.2793
            6    0.0218
            7    0.2518
            8    0.2928
            9    1.8181
            Name: x, dtype: float64
            -----------------------
    """
    if isinstance(close, pd.Series) and isinstance(m, int) & isinstance(n, int):
        close = close / close.shift(m) - 1
        close = close.rolling(n).apply(lambda ser: __get_s_risk_glvarianceratio20(ser_=ser))
        return close
    else:
        raise ValueError('传入的close需为Series格式,n, m需为int格式')


def __get_s_risk_glvarianceratio20(ser_):
    up_list = [x for x in ser_.tolist() if x > 0]
    down_list = [x for x in ser_.tolist() if x <= 0]
    if len(down_list) == 0 or len(up_list) == 0:
        return np.nan
    var_up = np.var(up_list)
    var_down = np.var(down_list)
    glvarianceratio20 = var_up / var_down
    return glvarianceratio20


# -*- coding:utf-8 -*-
# @Time: 2020/11/8 16:44
# @Author: jiangwenhao
# @File: max_vol.py

import pandas as pd
import numpy as np


def cal(vol):
	"""
	计算个股今日最大量

	Parameters
	----------
	vol : Series
		成交量
	Returns
	-------
	float
		个股今日最大量
		Examples:
			----------------------------------
			302594.62
			----------------------------------
	"""
	if isinstance(vol, pd.Series):
		df_compare = pd.DataFrame({'vol': vol}).reset_index(drop=True)
		max_vol = df_compare['vol'].max()
	else:
		raise ValueError('传入的vol需为Series格式')
	return max_vol



# -*- coding:utf-8 -*-
# @Time: 2020/11/10 13:30
# @Author: caiyucheng
# @File: cross_n_min_amount.py

import pandas as pd
import numpy as np


def cal(open, high, low, close, n):
    """
    相对离散指数(n期)
    Co=close-open;
    HL = high-low;
    V1=(co+2*ref(co,1)+2*ref(co,2)+ref(co,3))/6;
    V2=(HL+2*ref(HL,1)+2*ref(HL,2)+ref(HL,3))/6;
    S1=sum(V1,n/2)
    S2=sum(v2,n/2)
    RVI=S1/S2
    RVIS=( RVI+2*ref(RVI,1)+2*ref(RVI,2)+ref(RVI,3))/6;
    其中，ref表示引用以前的数据，sum表示累计和，n表示周期（基本设置n/2为１０），RVIS是RVI的信号线
    在计算时，先计算收盘价与开盘价的差值，最高价与最低价的差值，然后把它们分别求４天的加权平均，
    然后分别求它们一半周期的累计和，再把累计和相除。RVI的信号线就是RVI的４天的加权平均值。
    VALUE1 = ((CLOSE - OPEN) + 2 * (CLOSE (1)) – OPEN (1)) + 2*(CLOSE (2) – OPEN (2)) + (CLOSE (3) – OPEN (3))) / 6
    VALUE2 = ((HIGH - LOW) + 2 * (HIGH (1) – LOW (1)) + 2*(HIGH (2)- LOW (2)) + (HIGH (3) – LOW (3))) / 6
    NUM = SUM (VALUE1, N)
    DENUM = SUM (VALUE2, N)
    RVI = NUM / DENUM
    RVISig = (RVI + 2 * RVI (1) + 2 * RVI (2) + RVI (3)) / 6  这个暂时没求
    Parameters
    ----------
    open: Series
          开盘价

    high: Series
          最高价
    low: Series
          最低价
    close: Series
          收盘价
    n: int
       计算区间
    Returns
    --------
    pd.Series
          Examples
          -------
            0        NaN
            1        NaN
            2        NaN
            3        NaN
            4        NaN
            5        NaN
            6        NaN
            7        NaN
            8        NaN
            9        NaN
            10       NaN
            11       NaN
            12   -0.1670
            13   -0.2304
            14   -0.2575
            15   -0.2599
            16   -0.2489
            17   -0.2229
            18   -0.1821
            19   -0.1203
            Name: x, dtype: float64
            -----------------------
    """
    if isinstance(open, pd.Series) and isinstance(close, pd.Series) and isinstance(high, pd.Series) and isinstance(low, pd.Series) and isinstance(n, int):
        co = close - open
        hl = high - low
        v1 = (co + 2 * co.shift(1) + 2 * co.shift(2) + co.shift(3)) / 6
        v2 = (hl + 2 * hl.shift(1) + 2 * hl.shift(2) + hl.shift(3)) / 6
        s1 = v1.rolling(window=n).sum()
        s2 = v2.rolling(window=n).sum()
        rvi = s1 / s2
        return rvi
    else:
        raise ValueError('请输入正确的参数格式, close, high,low, open均为Series, n为int')












# -*- coding:utf-8 -*-
# @Time: 2020/11/7 15:09
# @Author: wangtao
# @File: firstcrosstime.py

import pandas as pd
import numpy as np
import datetime


def cal(time: pd.Series, price: pd.Series, target_price):
	"""
	突破一定涨幅的时间分钟数
	例如：可用来计算涨停时间
	
	Parameters
	----------
	time : Series
		当天的分钟序列
	price : Series
		当天分钟价格序列
	target_price : float or Series
		昨日收盘价（例如用于计算涨停价格）
		Examples1:
			8.91
		Examples2:
			----------------------------------
			0      8.91
			1      8.91
			...
			238    8.91
			239    8.91
			Name: pre_close, Length: 240, dtype: float64
			----------------------------------
	time_start : str, Default '09:30:00'
		计时起始分钟
		
	Returns
	-------
	float
		首次突破时间（例如首次涨停时间）
		Examples:
			----------
			48.0
			----------
	"""
	
	if isinstance(time, pd.Series) and isinstance(price, pd.Series):
		df_merge = pd.DataFrame({'time': time, 'price': price})
		df_merge['target_price'] = target_price
		df_cross = df_merge[round(df_merge['price'], 2) >= round(df_merge['target_price'], 2)]
		mins = df_cross.index.min() + 1
	else:
		raise ValueError('传入的time和high需为Series格式')
	return mins


if __name__ == '__main__':
	import qytools
	db_read = qytools.db_read.DBReader()
	date = db_read.get_opendate(20200826)
	df_min_date = db_read.read_tdx_1min_data(start=date, end=date,fields='code,date,time,close', code=[601216])
	df_day_date = db_read.read_ts_day_data(start=20200826, end=20200826, fields='code,date,up_limit', code=[601216])
	df_min_date['up_limit'] = df_day_date.up_limit[0]
	cal(df_min_date.time, df_min_date.close, df_min_date.up_limit)



# -*- coding:utf-8 -*-
# @Time: 2020/11/9 10:03
# @Author: jiangwenhao
# @File: cross_n_min_amount.py

import pandas as pd
import numpy as np


def cal(close, up_limit, amount, n=1) -> pd.Series:
	"""
	计算个股今日涨停前n分钟的成交额

	Parameters
	----------
	close : Series
		收盘价
	up_limit : Series
		涨停价（index应与close对应）
	amount : Series
		成交量
	n : int
		突破前n分钟，默认为1

	Returns
	-------
	pd.Series
		个股今日涨停前n分钟的成交额
		Examples:
			----------------------------------
			0            NaN
			1            NaN
			2            NaN
			3            NaN
			4            NaN
			...
			82           NaN
			83           NaN
			84           NaN
			85           NaN
			86    309095.982
			Name: cross_n_min_amount, Length: 87, dtype: float64
			----------------------------------
	"""
	if isinstance(close, pd.Series) and isinstance(up_limit, pd.Series) & isinstance(amount, pd.Series) & isinstance(n, int):
		df_compare = pd.DataFrame({'close': close, 'up_limit': up_limit, 'amount': amount}).reset_index(drop=True)
		df_compare['last_min_amount'] = df_compare['amount'].shift(1)
		df_compare['last_1_min_close'] = df_compare['close'].shift(1)
		df_compare['is_cross'] = df_compare.apply(
			lambda x: 1 if
			(round(x['close'], 2) >= round(x['up_limit'], 2)) & (round(x['last_1_min_close'], 2) < round(x['up_limit'], 2))
			else 0, axis=1
		)
		df_compare['n_min_amount'] = df_compare['last_min_amount'].rolling(window=n, min_periods=1).sum()
		cross_n_min_amount = df_compare.apply(
			lambda x: x['n_min_amount'] if x['is_cross'] == 1 else np.nan, axis=1
				)
		cross_n_min_amount = cross_n_min_amount.dropna()
		if len(cross_n_min_amount) == 0:
			cross_n_min_amount = pd.Series([np.nan])
	else:
		raise ValueError('传入的close,up_limit和amount需为Series格式,n需为int格式')
	return cross_n_min_amount



# -*- coding:utf-8 -*-
# @Time: 2020/11/9 09:16
# @Author: jiangwenhao
# @File: rise_speed_n_min.py

import pandas as pd
import numpy as np


def cal(close, up_limit, n=1):
	"""
	计算突破涨停前n分钟的涨速

	Parameters
	----------
	close : Series
		收盘价
	up_limit : Series
		涨停价（index应与close对应）
	n: int
		突破前n分钟，默认为1
	Returns
	-------
	pd.Series or np.array
		突破涨停前n分钟的涨速
		Examples:
			----------------------------------
			0    1.445378
			Name: rise_speed_n_min, Length: 1, dtype: float64
			----------------------------------
	"""
	if isinstance(close, pd.Series) and isinstance(up_limit, pd.Series) and isinstance(n, int):
		df_compare = pd.DataFrame({'close': close, 'up_limit': up_limit}).reset_index(drop=True)
		df_compare['last_1_min_close'] = df_compare['close'].shift(1)
		df_compare['is_cross'] = df_compare.apply(
			lambda x: 1 if
			(round(x['close'], 2) >= round(x['up_limit'], 2)) & (round(x['last_1_min_close'], 2) < round(x['up_limit'], 2))
			else 0, axis=1
		)
		df_compare['last_n_min_close'] = df_compare['close'].shift(n)

		rise_speed_n_min = df_compare.apply(
			lambda x: (x['up_limit']-x['last_n_min_close'])/x['last_n_min_close'] * 100
			if x['is_cross'] == 1
			else np.nan, axis=1
		).dropna().reset_index(drop=True)
		if len(rise_speed_n_min) ==0 :
			rise_speed_n_min =pd.Series([np.nan])
	else:
		raise ValueError('传入的close和up_limit需为Series格式,n需要为int格式')
	return rise_speed_n_min



# -*- coding:utf-8 -*-
# @Time: 2020/11/10 19:05
# @Author: caiyucheng
# @File: cross_n_min_amount.py

import pandas as pd
import numpy as np


def cal(net_mf_vol, vol):
    """
    流入率 net_mf_vol / vol
    Parameters
    ----------
    net_mf_vol: pd.Series
                净流入量
    vol: pd.Series
        成交量
    Returns
    -------
    pd.Series
        Examples
        --------
            0    0.3113
            1    0.2635
            2    0.2766
            3    0.2398
            4    0.2290
            5    0.2919
            6    0.2817
            7    0.2575
            8    0.2411
            9    0.2305
            Name: x, dtype: float64
            -----------------------
    """
    if isinstance(net_mf_vol, pd.Series) and isinstance(vol, pd.Series):
        return net_mf_vol / vol
    else:
        raise ValueError('请输入格式正确的参数, net_mf_vol, vol均为Series')



# -*- coding:utf-8 -*-
# @Time: 2020/11/10 19:05
# @Author: caiyucheng
# @File: cross_n_min_amount.py

import pandas as pd
import numpy as np


def cal(buy_elg_amount, sell_elg_amount):
    """
    机构金额差 buy_elg_amount - sell_elg_amount
    Parameters
    ----------
    buy_elg_amount: pd.Series
                    特大单买入金额
    sell_elg_amount: pd.Series
                    特大单卖出金额
    Returns
    -------
        pd.Series
        Examples
        --------
            0   -21122.18
            1   -33742.42
            2    -2165.09
            3     6412.60
            4    -3999.69
            5    -4694.99
            6     8225.93
            7   -21593.76
            8   -27117.57
            9    -2200.52
            Name: x, dtype: float64
            -----------------------
    """
    if isinstance(buy_elg_amount, pd.Series) and isinstance(sell_elg_amount, pd.Series):
        return buy_elg_amount - sell_elg_amount
    else:
        raise ValueError('请输入格式正确的参数, buy_elg_amount, sell_elg_amount均为Series')
